module ActivityPub
  module Rails::ActorConcern
    extend ActiveSupport::Concern

    USERNAME_REGEXP = /[a-z0-9_]+([a-z0-9_\.]+[a-z0-9_]+)?/i
    MENTION_REGEXP  = /(?<=^|[^\/[:word:]])@((#{USERNAME_REGEXP})(?:@[a-z0-9\.\-]+[a-z0-9]+)?)/i

    included do
      include ActivityPub::Rails::ObjectConcern

      scope :matches_slug, ->(value) { where(arel_table[:slug].matches("#{value}%")) }
      scope :matches_display_name, ->(value) { where(arel_table[:display_name].matches("#{value}%")) }

      before_create :generate_keys

      validates :slug, presence: true
      validates :slug, uniqueness: { scope: :domain, case_sensitive: true },
                       if: -> { !local? }
      validates :slug, format: { with: /\A[a-z0-9_]+\z/i },
                       length: { maximum: 30 }, if: -> { local? },
                       uniqueness: { scope: :domain, case_sensitive: true }
    end

    def acct
      local? ? slug : "#{slug}@#{domain}"
    end

    def possibly_stale?
      last_webfingered_at.nil? || last_webfingered_at <= 1.day.ago
    end

    def local_slug_and_domain
      "#{slug}@#{ActivityPub.local_domain}"
    end

    def to_webfinger_s
      "acct:#{local_slug_and_domain}"
    end

    def keypair
      @keypair ||= OpenSSL::PKey::RSA.new(private_key || public_key)
    end

    def magic_key
      modulus, exponent = [keypair.public_key.n, keypair.public_key.e].map do |component|
        result = []

        until component.zero?
          result << [component % 256].pack('C')
          component >>= 8
        end

        result.reverse.join
      end

      (['RSA'] + [modulus, exponent].map { |n| Base64.urlsafe_encode64(n) }).join('.')
    end

    private

    def generate_keys
      return unless local? && !Rails.env.test?

      keypair = OpenSSL::PKey::RSA.new(2048)
      self.private_key = keypair.to_pem
      self.public_key  = keypair.public_key.to_pem
    end
  end
end
